.panels {
  margin-top: -24px;
  margin-left: -16px;
}
.panel {
  margin-top: 24px;
  margin-left: 16px;
  min-width: 300px;
  flex: 1 1 0;
  display: flex;
  flex-direction: column;
  box-sizing: border-box;
}
.panel_header {
  text-transform: uppercase;
  color: black;
  font-size: 14px;
  font-weight: bold;
  display: flex;
  margin-bottom: 8px;
}
.input_interfaces, .output_interfaces {
  margin-bottom: 16px;
  background-color: whitesmoke;
  border-radius: 4px;
  padding: 12px;
  flex-grow: 1;
}
.interface {
  display: flex;
  flex-flow: column;
}
.interface_box {
  height: 360px;
}
.interface_mini_box {
  height: 180px;
}
.interface_max_box {
  overflow: auto;
  max-height: 360px;
}
.interface:not(*:last-child) {
  margin-bottom: 16px;
}
.output_panel {
  position: relative;
}
.loading {
  position: absolute;
  top: 3px;
  right: 3px;
  margin-left: auto;
  z-index: 1;
}
.loading img {
  height: 20px;
  display: none;
}
.panel_buttons {
  display: flex;
  margin-left: -16px;
}
button.submit {
  display: none;
}
.panel_button {
  background-color: whitesmoke;
  padding: 12px;
  box-sizing: border-box;
  font-weight: bold;
  border: 0 none;
  border-radius: 4px;
  margin-left: 16px;
  flex-grow: 1;
  text-align: center;
}
.panel_button.left_panel_button {
  border-top-right-radius: 0;  
  border-bottom-right-radius: 0;  
  flex-grow: 0.6;
}
.panel_button.right_panel_button {
  border-top-left-radius: 0;  
  border-bottom-left-radius: 0;  
  flex-grow: 0.6;
  margin-left: 0px;
  background-color: #EEE;
}
.record_stop {
  display: inline-flex;
  align-items: center;
  justify-content: center;
  width: 60px;
  border-radius: 4px;
}
.record_square {
  width: 20px; 
  height: 20px;
  background-color: #c90a0a;
  box-shadow: inset 0 0 4px darkred;
}
button.submit {
  background-color: #e67e22;
  color: white;
}
.panel_button.inactive {
  color: lightgray;
  cursor: not-allowed;
}
.panel_button:not(.inactive):hover {
  background-color: lightgray;
}
button.submit:hover {
  background-color: #f39c12 !important;
}
.flag {
  cursor: pointer;
  position: relative;
  transition: background-color 100ms;
}
.flagged {
  background-color: #e74c3c !important;
}
.dropdown:after {
  content: '';
  border: 4px solid transparent;
  border-top: 4px solid black;
  margin-left: 8px;
  margin-bottom: 3px;
  display: inline-block;
  vertical-align: bottom;
}
.dropcontent {
  color: black;
  display: none;
  position: absolute;
  z-index: 1;
  background-color: white;
  box-shadow: 0px 4px 8px 0px lightgray;   
  min-width: 170px;
  right: 0px;
  top: 44px;
}
.dropcontent div {
  display: block;
  margin: 4px;
  padding: 8px;
}
.dropcontent div:hover {
  background-color: lightgray;
}
.dropdown:hover .dropcontent {
  display: block;
}
.overlay {
  position: absolute;
  height: 100vh;
  width: 100vw;
  position: fixed;
  z-index: 1;
  background-color: rgba(0, 0, 0, 0.7);
  top: 0;
  left: 0;
}
.loading {
  justify-content: center;
  align-items: center;
}

.hidden {
  display: none !important;
}
.screenshot_logo {
  display: flex;
  justify-content: center;
  flex-grow: 1;
  padding: 4px;
  box-sizing: border-box;
  margin-left: 16px;
}
.screenshot_logo img {
  height: 38px;
}
#login {
  margin: 40px auto;
  width: 50%;
  background-color: whitesmoke;
  padding: 24px;
  border-radius: 4px;
  display: flex;
  flex-direction: column;
}
#login h2 {
  margin-top: 8px;
  margin-bottom: 8px
}
#login label  {
  display: block;
  margin: 0.5em 0;
}
#login input[type=text], #login input[type=password], #login textarea {
  background-color: white;
  color: inherit;
  border: none;
  padding: 0;
  font: inherit;
  font-size: 1.1em;
  outline: inherit;
  padding: 0.5em;
}
#login input[type="submit"] {
  padding: 12px;
  margin-top: 20px;
  background-color: #e67e22;
  color: white;  
  border-radius: 4px;
  font-weight: bold;
}
